package com.myoa.engineering.crawl.shopping.configuration.resilience;

import io.github.resilience4j.circuitbreaker.CircuitBreaker;
import io.github.resilience4j.circuitbreaker.CircuitBreakerConfig;
import io.github.resilience4j.circuitbreaker.CircuitBreakerRegistry;
import io.github.resilience4j.core.RegistryStore;
import io.github.resilience4j.core.registry.InMemoryRegistryStore;
import io.github.resilience4j.ratelimiter.RateLimiter;
import io.github.resilience4j.ratelimiter.RateLimiterConfig;
import io.github.resilience4j.ratelimiter.RateLimiterRegistry;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.time.Duration;

@Configuration
public class RateLimitConfiguration {

    @Bean
    public RateLimiterRegistry rateLimiterRegistry() {
        RegistryStore<RateLimiter> stores = new InMemoryRegistryStore<>();

        // TODO 개별 config 에서 등록하도록 변경
        RateLimiterConfig rateLimiterConfig = RateLimiterConfig.custom()
                                                               .limitRefreshPeriod(Duration.ofMillis(500)) // 0.5 seconds
                                                               .limitForPeriod(1) // number of permits in a refresh period
                                                               .build();
        stores.putIfAbsent("fmkoreaAvoid429", RateLimiter.of("fmkoreaAvoid429", rateLimiterConfig));

        return RateLimiterRegistry.custom()
                                  .withRateLimiterConfig(RateLimiterConfig.ofDefaults())
                                  .withRegistryStore(stores)
                                  .build();
    }

    @Bean
    public CircuitBreakerRegistry circuitBreakerRegistry() {
        RegistryStore<CircuitBreaker> stores = new InMemoryRegistryStore<>();

        CircuitBreakerConfig circuitBreakerConfig = CircuitBreakerConfig.custom()
                                                                        .slidingWindowSize(1)
                                                                        .build();
        stores.putIfAbsent("fmkoreaAvoid429", CircuitBreaker.of("fmkoreaAvoid429", circuitBreakerConfig));

        return CircuitBreakerRegistry.custom()
                                     .withCircuitBreakerConfig(CircuitBreakerConfig.ofDefaults())
                                     .withRegistryStore(stores)
                                     .build();

    }

}
