package com.myoa.engineering.crawl.shopping.controller;

import com.myoa.engineering.crawl.shopping.dto.slack.v2.SlackEventSubscriptionDTO;
import com.myoa.engineering.crawl.shopping.dto.slack.v2.SlackSlashCommandDTO;
import com.myoa.engineering.crawl.shopping.service.slack.RegisterCommandHandler;
import com.myoa.engineering.crawl.shopping.service.slack.ShoppingCommandHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api/v1/slack")
public class SlackEventSubscriptionAPIController {

    private final ShoppingCommandHandler shoppingCommandHandler;
    private final RegisterCommandHandler registerCommandHandler;

    public SlackEventSubscriptionAPIController(ShoppingCommandHandler shoppingCommandHandler,
                                               RegisterCommandHandler registerCommandHandler) {
        this.shoppingCommandHandler = shoppingCommandHandler;
        this.registerCommandHandler = registerCommandHandler;
    }

    @PostMapping(value = {"/subscriptions"})
    private ResponseEntity<String> slackEventSubscription(@RequestBody SlackEventSubscriptionDTO requestDTO) {
        log.info("requestDTO: {}", requestDTO);
        return ResponseEntity.ok(requestDTO.getChallenge());
    }

    @PostMapping(value = {"/commands/help"}, consumes = {"application/x-www-form-urlencoded"})
    private ResponseEntity<String> handleCommandHelp(SlackSlashCommandDTO requestDTO) {
        log.info("requestDTO: {}", requestDTO);
        return ResponseEntity.ok("""
                                         /쇼핑 목록
                                         /쇼핑 키워드 추가 (쇼핑몰명) (키워드명)
                                         /쇼핑 키워드 삭제 (쇼핑몰명) (키워드명)
                                         /쇼핑 키워드 목록
                                         /등록 (사용자명)
                                         """);
    }

    @PostMapping(value = {"/commands/shopping"}, consumes = {"application/x-www-form-urlencoded"})
    private ResponseEntity<String> handleAddSubscription(SlackSlashCommandDTO requestDTO) {
        log.info("requestDTO: {}", requestDTO);

        List<String> results = shoppingCommandHandler.handle(requestDTO);

        return ResponseEntity.ok(String.join("\n", results));
    }

    @PostMapping(value = {"/commands/register"}, consumes = {"application/x-www-form-urlencoded"})
    private ResponseEntity<String> handleRegisterUser(SlackSlashCommandDTO requestDTO) {
        log.info("requestDTO: {}", requestDTO);

        List<String> results = registerCommandHandler.handle(requestDTO);

        return ResponseEntity.ok(String.join("\n", results));
    }
}
