package com.myoa.engineering.crawl.shopping.crawlhandler;

import com.myoa.engineering.crawl.shopping.crawlhandler.parser.FmkoreaArticleParser;
import com.myoa.engineering.crawl.shopping.domain.entity.v2.Article;
import com.myoa.engineering.crawl.shopping.infra.client.fmkorea.FmkoreaBoardClientV2;
import com.myoa.engineering.crawl.shopping.service.ArticleCommandService;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Stream;

@Slf4j
@Component
public class FmkoreaCrawlHandler implements CrawlHandler {

    private final FmkoreaBoardClientV2 fmkoreaBoardClientV2;
    private final FmkoreaArticleParser fmkoreaArticleParser;
    private final ArticleCommandService articleCommandService;
    private final FmkoreaCookieService fmkoreaCookieService;

    public FmkoreaCrawlHandler(FmkoreaBoardClientV2 fmkoreaBoardClientV2,
                               FmkoreaArticleParser fmkoreaArticleParser, ArticleCommandService articleCommandService, FmkoreaCookieService fmkoreaCookieService) {
        this.fmkoreaBoardClientV2 = fmkoreaBoardClientV2;
        this.fmkoreaArticleParser = fmkoreaArticleParser;
        this.articleCommandService = articleCommandService;
        this.fmkoreaCookieService = fmkoreaCookieService;
    }

    @Override
    public CrawlTarget getCrawlTarget() {
        return CrawlTarget.FMKOREA;
    }

    @Override
    public void handle() {
        String cookie = fmkoreaCookieService.getCookie();

        String boardHtmlPage1 = fmkoreaBoardClientV2.getBoardHtml(1, cookie);
        List<Article> parsedPage1 = fmkoreaArticleParser.parse(boardHtmlPage1);

        String boardHtmlPage2 = fmkoreaBoardClientV2.getBoardHtml(2, cookie);
        List<Article> parsedPage2 = fmkoreaArticleParser.parse(boardHtmlPage2);

        List<Article> merged = Stream.of(parsedPage1, parsedPage2)
                                     .flatMap(List::stream)
                                     .map(e -> e.updateCrawlTarget(getCrawlTarget()))
                                     .toList();

        articleCommandService.upsert(merged);
    }

}
