package com.myoa.engineering.crawl.shopping.crawlhandler.parser;

import com.myoa.engineering.crawl.shopping.domain.entity.v2.Article;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import com.myoa.engineering.crawl.shopping.util.DateTimeUtils;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Component;

import java.time.ZonedDateTime;
import java.util.List;

@Slf4j
@Component
public class FmkoreaArticleParser {

    private static final String FMKOREA_URL = "https://www.fmkorea.com";

    public List<Article> parse(String html) {
        Elements liElements = converHtmlToTrElements(html);
        return liElements.stream()
//                         .filter(this::isRealArticle)
                         .map(this::parse)
                         .toList();
    }

    private Elements converHtmlToTrElements(String html) {
        Document document = Jsoup.parse(html);
        Element liTable = document.getElementsByClass("fm_best_widget").first();
        return liTable.select("li.li_best2_pop0");
    }

    private Article parse(Element item) {
        String link = item.select("h3.title a").attr("href");
        Long articleId = Long.parseLong(link.replace("/", ""));

        // https://www.fmkorea.com/7023440365
        String articleUrl = FMKOREA_URL + link;
        String boardName = item.select("span.category a").text().trim();

        String title = item.select("h3.title a").text();
        String itemPrice = item.select("div.hotdeal_info span:contains(가격) a").text();
        String deliveryPrice = item.select("div.hotdeal_info span:contains(배송) a").text();
        title = title + " 가격: " + itemPrice + " 배송: " + deliveryPrice;
        String registeredAtString = item.select("span.regdate").text().trim();
        ZonedDateTime registeredAt = DateTimeUtils.parse(registeredAtString, DateTimeUtils.FORMATTER_HHMM, DateTimeUtils.FORMATTER_YYMMDD_DOT);

        Element recommendationElement = item.selectFirst("a.pc_voted_count");
        Integer recommended = null;
        if (recommendationElement != null) {
            recommended = Integer.parseInt(recommendationElement.selectFirst("span.count").text());
        }

        return Article.builder()
                      .articleId(articleId)
                      .title(title)
                      .boardName(boardName)
                      .articleUrl(articleUrl)
                      .recommended(recommended)
                      .registeredAt(registeredAt)
                      .build();
    }


}
