package com.myoa.engineering.crawl.shopping.service;

import com.myoa.engineering.crawl.shopping.domain.entity.v2.SubscribedKeyword;
import com.myoa.engineering.crawl.shopping.dto.feed.v1.SubscribedKeywordDTO;
import com.myoa.engineering.crawl.shopping.infra.repository.v2.SubscribedKeywordRepository;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class SubscribedKeywordCommandService {

    private final SubscribedKeywordRepository subscribedKeywordRepository;

    public SubscribedKeywordCommandService(SubscribedKeywordRepository subscribedKeywordRepository) {
        this.subscribedKeywordRepository = subscribedKeywordRepository;
    }

    @Transactional
    public void add(SubscribedKeyword subscribedKeyword) {
        subscribedKeywordRepository.save(subscribedKeyword);
    }

    @Transactional
    public void delete(String userId, CrawlTarget crawlTarget, String keyword) {
        SubscribedKeyword subscribedKeyword = subscribedKeywordRepository.findByUserIdAndCrawlTargetAndKeyword(userId, crawlTarget, keyword)
                                                                         .orElseThrow(() -> new IllegalArgumentException("Not found keyword: " + keyword));

        subscribedKeywordRepository.delete(subscribedKeyword);
    }

    @Transactional(readOnly = true)
    public List<SubscribedKeywordDTO> list(String userId) {
        return subscribedKeywordRepository.findByUserId(userId)
                                          .stream()
                                          .map(e -> SubscribedKeywordDTO.builder()
                                                                        .keyword(e.getKeyword())
                                                                        .userId(e.getUserId())
                                                                        .crawlTarget(e.getCrawlTarget())
                                                                        .build())
                                          .toList();
    }

}
