package com.myoa.engineering.crawl.shopping.service.slack;

import com.myoa.engineering.crawl.shopping.dto.slack.v2.SlackSlashCommandDTO;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ShoppingCommandHandler implements SlackCommandHandler {

    private final List<ShoppingCommandProcessor> processors;

    public ShoppingCommandHandler(List<ShoppingCommandProcessor> processors) {
        this.processors = processors;
    }

    public List<String> handle(SlackSlashCommandDTO requestDTO) {
        String[] commands = requestDTO.getText().split("\\s+", 2);

        if (!requestDTO.getCommand().equals("/쇼핑")) {
            throw new IllegalArgumentException("Invalid command: " + requestDTO.getCommand());
        }

        final String action = commands[0];

        return processors.stream()
                         .filter(processor -> processor.isAssignable(action))
                         .map(processor -> processor.process(requestDTO))
                         .toList();

    }


}
